import numpy as np
import matplotlib as mpl
mpl.use('pgf')
import matplotlib.pyplot as plt
from aggregates import *
from cement_paste import *
from mori_tanaka import *
from blending import *


def figsize(scale):
    fig_width_pt = 469.755                          # get this from LaTeX using \the\textwidth
    inches_per_pt = 1.0/72.27                       # convert pt to inch

    height_multiplier = 1.1  # (np.sqrt(5.0)-1.0)/2.0

    fig_width = fig_width_pt*inches_per_pt*scale    # width in inches
    fig_height = fig_width*height_multiplier
    fig_size = [fig_width,fig_height]
    return fig_size


pgf_with_latex = {                      # setup matplotlib to use latex for output
    "pgf.texsystem": "pdflatex",        # change this if using xetex or latex
    "text.usetex": True,                # use LaTeX to write all text
    "font.family": "serif",
    "font.serif": [],                   # blank entries should cause plots to inherit fonts from the document
    "font.sans-serif": [],
    "font.monospace": [],
    "axes.labelsize": 11,               # LaTeX default is 10pt font.
    "font.size": 11,
    "legend.fontsize": 11,               # Make the legend/label fonts a little smaller
    "xtick.labelsize": 10,
    "ytick.labelsize": 10,
    "figure.figsize": figsize(0.9),     # default fig size of 0.9 textwidth
    "pgf.preamble": [
        r"\usepackage[utf8x]{inputenc}",    # use utf8 fonts becasue your computer can handle it :)
        r"\usepackage[T1]{fontenc}",        # plots will be generated using this preamble
        r"\usepackage{upgreek}",
        ]
    }
mpl.rcParams.update(pgf_with_latex)

import matplotlib.pyplot as plt

# I make my own newfig and savefig functions
def newfig(width):
    plt.clf()
    fig = plt.figure(figsize=figsize(width))
    ax = fig.add_subplot(111)
    return fig, ax

def savefig(filename):
    plt.savefig('{}.pgf'.format(filename), bbox_inches='tight')
    plt.savefig('{}.pdf'.format(filename), bbox_inches='tight')


# settings (direct tensile test simulation)
# d_eps = 1e-5  # strain increment in the analysis
# max_eps = 5e-3  # maximum strain reached in the analysis
# max_omega_error = 1e-8  # iteration criterion (max omega error)


# LEVEL I
# low density, high density C-S-H gels to create matrix
c = [0.706, 0.294]
e = [20.090e9, 33.930e9]
nu = [0.200, 0.200]
e_eff_csh, nu_eff_csh = mori_tanaka(c, e, nu)


# LEVEL II
# variable inputs
aggregates_mass_percentages = [50.0, 25.0, 20.0, 15.0, 10.0, 5.0, 0.0]
e_eff_meas = [15.437e9, 19.686e9, 20.827e9, 21.567e9, 23.489e9, 24.386e9, 25.820e9]
e_meas_std = [0.710e9, 1.116e9, 0.646e9, 0.861e9, 0.667e9, 0.430e9, 0.600e9]
fc_meas = [51.7e6, 62.1e6, 63.2e6, 65.4e6, 67.0e6, 71.1e6, 74.5e6]
fc_meas_std = [3.37e6, 4.06e6, 1.99e6, 2.12e6, 4.94e6, 4.24e6, 3.52e6]
porosities = [0.19878, 0.16740, 0.16635, 0.16091, 0.15590, 0.14965, 0.13869]  # in % of volume

# get material inputs
aggregates = input_aggregates()
cement_paste = input_cement_paste()

e_eff_calc, fc_calc, fc_arel = [], [], []
for idx, aggregate_mass_percentage in enumerate(aggregates_mass_percentages):
    porosity = porosities[idx]  

    # calculation of compressive strength (J2 criterion for the matrix)
    sigma_0 = np.zeros((3, 3))
    sigma_0[0][0] = cement_paste['matrix_comp_strength']
    sigma_dev_0 = sigma_0-1/3*np.trace(sigma_0)*np.eye(3)
    j2_0 = np.sqrt(3/2*sum(sum(np.multiply(sigma_dev_0, sigma_dev_0))))
    strength_test = 1e-5  # "test" stress to be scaled
    sigma_test = np.array([[strength_test], [0], [0], [0], [0], [0]])

    # C-S-H matrix, clinker, portlandite (CH), low stiffness phases, additive, porosity
    [c_matrix, c_uncoated, c_coated, c_coating, e_matrix, e_uncoated, e_coated, e_coating, nu_matrix, nu_uncoated,
     nu_coated, nu_coating, r_coated, r_coating] = get_components([aggregate_mass_percentage], cement_paste, aggregates,
                                                                  porosity, e_eff_csh, nu_eff_csh)

    # homogenization at Level II
    k_eff, g_eff, a_dil_k, a_dil_g, a_mt_k, a_mt_g = mori_tanaka_with_coated_particles(c_matrix, c_uncoated, c_coated,
                                                                                       c_coating, e_matrix, e_uncoated,
                                                                                       e_coated, e_coating, nu_matrix,
                                                                                       nu_uncoated, nu_coated,
                                                                                       nu_coating, r_coated, r_coating)
    e_eff, nu_eff = get_e_nu_from_k_g(k_eff, g_eff)

    # calculate Mori-Tanaka concentration factor and stress in matrix
    A_MT = A_from_ak_and_ag(a_mt_k, a_mt_g)
    k_matrix, g_matrix = get_k_g_from_e_nu(e_matrix, nu_matrix)
    L_matrix = L_from_k_and_g(k_matrix, g_matrix)
    L_eff = L_from_k_and_g(k_eff, g_eff)
    B_MT = np.dot(np.dot(L_matrix, A_MT), np.linalg.inv(L_eff))
    s0 = np.dot(B_MT, sigma_test)
    stressInMatrix_tensor = np.array([[s0[0][0], s0[5][0], s0[4][0]],
                                      [s0[5][0], s0[1][0], s0[3][0]],
                                      [s0[4][0], s0[3][0], s0[2][0]]])
    sigma_dev_matrix = stressInMatrix_tensor-1/3*np.trace(stressInMatrix_tensor)*np.eye(3)
    j2_matrix = np.sqrt(3/2*sum(sum(np.multiply(sigma_dev_matrix, sigma_dev_matrix))))
    fc = j2_0/j2_matrix*strength_test

    # save the results
    e_eff_calc.append(e_eff)
    fc_calc.append(fc)

    # calculate fc by Arel's formula
    b, c, d, e = aggregate_mass_percentage, 2.77, 92.5, 0.26
    fc_arel.append(68.3-13.0+np.tan(0.74*c)-np.sin(b*e)-0.5*b-34.06*np.cos(-5.73*d))

# PLOTTING
# stiffness
fig, ax = newfig(0.45)
e_eff_meas_plotting = [e_eff/1e9 for e_eff in e_eff_meas]
e_eff_calc_plotting = [e_eff/1e9 for e_eff in e_eff_calc]
e_meas_std_plotting = [std/1e9 for std in e_meas_std]
plt.plot(aggregates_mass_percentages, e_eff_meas_plotting, '-o', color='b', label='Measurements')
plt.errorbar(aggregates_mass_percentages, e_eff_meas_plotting, yerr=e_meas_std_plotting)
plt.plot(aggregates_mass_percentages, e_eff_calc_plotting, '-s', color='r', label='Calculations')
plt.legend(loc='upper right')
plt.xlim(xmin=-5, xmax=55)
ax.set_xlabel(r'Amount of marble powder ($w_{\mathrm{m}}$) [\%]')
ax.set_ylabel(r'Young\'s modulus [GPa]')
plt.grid(True)
savefig('output/modelStiffness')

# compressive strength
fig, ax = newfig(0.45)
fc_meas_plotting = [fc/1e6 for fc in fc_meas]
fc_calc_plotting = [fc/1e6 for fc in fc_calc]
# fc_arel_plotting = [fc/1e6 for fc in fc_arel]
fc_meas_std_plotting = [std/1e6 for std in fc_meas_std]
plt.plot(aggregates_mass_percentages, fc_meas_plotting, '-o', color='b', label='Measurements')
plt.errorbar(aggregates_mass_percentages, fc_meas_plotting, yerr=fc_meas_std_plotting)
plt.plot(aggregates_mass_percentages, fc_calc_plotting, '-s', color='r', label='Calculations')
plt.plot(aggregates_mass_percentages, fc_arel, '--d', color='black', label='Arel\'s formula' )
plt.legend(loc='upper right')
plt.xlim(xmin=-5, xmax=55)
ax.set_xlabel(r'Amount of marble powder ($w_{\mathrm{m}}$) [\%]')
ax.set_ylabel(r'Compressive strength [MPa]')
plt.grid(True)
savefig('output/modelStrength')