import numpy as np
from elasticity import *


def eshelby_spherical(nu):
    alpha = (1+nu)/(3*(1+nu))
    beta = (2*(4-5*nu))/(15*(1-nu))
    return alpha, beta


def dilute_factors(alpha, beta, k, g):
    a_k, a_g = [], []
    n_phases = len(k)
    for r in range(n_phases):
        a_k.append(k[0]/(k[0]+alpha*(k[r]-k[0])))
        a_g.append(g[0]/(g[0]+beta*(g[r]-g[0])))
    return a_k, a_g


def strain_factors_herve_zaoui(n, k, g, nu, r):
    '''
    Calculation of the dilute concentration factor for spherical inclusion
    with coating

    input: number of layers within the inclusion, n [-]
           bulk moduli k (in vectors, ordered from centre of inclusion, incl. matrix) [Pa]
           shear moduli g (in vectors, ordered from centre of inclusion, incl. matrix) [Pa]
           Poisson ratios nu (in vectors, ordered from centre of inclusion, incl. matrix) [-]
           radii (ordered from centre of inclusion) [m]
    output: constraint constants for shperical inhomogeneity and its coating(s)

    Written originally in MATLAB by Venca 16/04/2012
    '''

    # Hydrostatic part
    N = np.zeros((2, 2, n))
    Q = np.zeros((2, 2, n))

    for i in range(n):
        N[:, :, i] = 1/(3*k[i+1]+4*g[i+1])*np.array([[3*k[i]+4*g[i+1], 4*(g[i+1]-g[i])/r[i]**3],
                                                    [3*(k[i+1]-k[i])*r[i]**3, 3*k[i+1]+4*g[i]]])
    Q[:, :, 0] = N[:, :, 0]*1.0
    for i in range(1, n):
        Q[:, :, i] = np.dot(N[:, :, i], Q[:, :, i-1])

    F = np.zeros((n, 1))
    F[0] = 1/Q[0, 0, n-1]
    for i in range(1, n):
        F[i] = Q[0, 0, i-1]/Q[0, 0, n-1]
    TK = F*1.0

    # Deviatoric part
    M = np.zeros((4, 4, n))
    P = np.zeros((4, 4, n))
    a = np.zeros(n)
    b = np.zeros(n)
    c = np.zeros(n)
    d = np.zeros(n)
    e = np.zeros(n)
    f = np.zeros(n)
    alpha = np.zeros(n)

    for i in range(n):
        a[i] = g[i]/g[i+1]*(7+5*nu[i])*(7-10*nu[i+1])-(7-10*nu[i])*(7+5*nu[i+1])
        b[i] = 4*(7-10*nu[i])+g[i]/g[i+1]*(7+5*nu[i])
        c[i] = 7-5*nu[i+1]+2*g[i]/g[i+1]*(4-5*nu[i+1])
        d[i] = 7+5*nu[i+1]+4*g[i]/g[i+1]*(7-10*nu[i+1])
        e[i] = 2*(4-5*nu[i])+(g[i]/g[i+1])*(7-5*nu[i])
        f[i] = (4-5*nu[i])*(7-5*nu[i+1])-(g[i]/g[i+1])*(4-5*nu[i+1])*(7-5*nu[i])
        alpha[i] = g[i]/g[i+1]-1

    for i in range(n):
        M[:, :, i] = 1/(5*(1-nu[i+1]))*np.array([
            [c[i]/3, r[i]**2*(3*b[i]-7*c[i])/(5*(1-2*nu[i])), -12*alpha[i]/(r[i]**5), 4*(f[i]-27*alpha[i])/(15*(1-2*nu[i])*r[i]**3)],
            [0, (1-2*nu[i+1])*b[i]/(7*(1-2*nu[i])), -20*(1-2*nu[i+1])*alpha[i]/(7*r[i]**7), -12*alpha[i]*(1-2*nu[i+1])/(7*(1-2*nu[i])*r[i]**5)],
            [r[i]**5*alpha[i]/2, -r[i]**7*(2*a[i]+147*alpha[i])/(70*(1-2*nu[i])), d[i]/7, r[i]**2*(105*(1-nu[i+1])+12*alpha[i]*(7-10*(nu[i+1]))-7*e[i])/(35*(1-2*nu[i]))],
            [-5/6*(1-2*nu[i+1])*alpha[i]*r[i]**3, 7*(1-2*nu[i+1])*alpha[i]*r[i]**5/(2*(1-2*nu[i])), 0.0, (1-2*nu[i+1])*e[i]/(3*(1-2*nu[i]))]])

    P[:, :, 0] = M[:, :, 0]*1.0
    for i in range(1, n):
        P[:, :, i] = np.dot(M[:, :, i], P[:, :, i-1])

    A = np.zeros((n, 1))
    B = np.zeros((n, 1))
    A[0] = P[1, 1, n-1]/(P[0, 0, n-1]*P[1, 1, n-1]-P[0, 1, n-1]*P[1, 0, n-1])
    B[0] = -P[1, 0, n-1]/(P[0, 0, n-1]*P[1, 1, n-1]-P[0, 1, n-1]*P[1, 0, n-1])

    for i in range(1, n):
        W = np.dot((1/(P[1, 1, n-1]*P[0, 0, n-1]-P[0, 1, n-1]*P[1, 0, n-1])*P[:, :, i-1]),
                   np.array([[P[1, 1, n-1]], [-P[1, 0, n-1]], [0], [0]]))
        A[i] = W[0]*1.0
        B[i] = W[1]*1.0

    TG = np.zeros((n, 1))
    TG[0] = A[0]-21/5*r[0]**2/(1-2*nu[0])*B[0]
    for i in range(1, n):
        TG[i] = A[i]-21/5*(r[i]**5-r[i-1]**5)/((1-2*nu[i])*(r[i]**3-r[i-1]**3))*B[i]

    return TK, TG


def mori_tanaka(c, e, nu):
    n_phases = len(c)

    # get k, g
    k, g = [], []
    for r in range(n_phases):
        kr, gr = get_k_g_from_e_nu(e[r], nu[r])
        k.append(kr)
        g.append(gr)

    # get dilute concentration factors
    alpha, beta = eshelby_spherical(nu[0])
    a_dil_k, a_dil_g = dilute_factors(alpha, beta, k, g)

    # calculate effective bulk and shear moduli
    sum_1, sum_2, sum_3, sum_4 = 0.0, 0.0, 0.0, 0.0
    for r in range(1, n_phases):
        sum_1 += c[r]*k[r]*a_dil_k[r]
        sum_2 += c[r]*g[r]*a_dil_g[r]
        sum_3 += c[r]*a_dil_k[r]
        sum_4 += c[r]*a_dil_g[r]
    k_eff = (c[0]*k[0]+sum_1)/(c[0]+sum_3)
    g_eff = (c[0]*g[0]+sum_2)/(c[0]+sum_4)

    # calculate effective Young's modulus and Poisson's ratio
    e_eff, nu_eff = get_e_nu_from_k_g(k_eff, g_eff)

    return e_eff, nu_eff


def mori_tanaka_with_coated_particles(c_matrix, c_uncoated, c_coated, c_coating, E_matrix, E_uncoated, E_coated,
                                      E_coating, nu_matrix, nu_uncoated, nu_coated, nu_coating, r_coated, r_coating):
    '''
    Calculates effective stiffness (K_eff and G_eff) of the composite containing coated particles

    input: c - volumetric fractions [-]
               - uncoated = uncoated particles
               - coated = coated particles
               - coating = corresponding coating of coated particles
           K - bulk modulus [Pa]
           G - shear modulus [Pa]
           r - radius of particle and coating, respectively [m]

    output: effective bulk modulus K_eff [Pa]
            effective shear modulus G_eff [Pa]

    originally written in MATLAB by Venca 30/04/2012
    '''

    # convert inputs
    K_matrix, G_matrix = get_k_g_from_e_nu(E_matrix, nu_matrix)
    K_uncoated, K_coated, K_coating, G_uncoated, G_coated, G_coating = [], [], [], [], [], []
    for i in range(len(c_uncoated)):
        k, g = get_k_g_from_e_nu(E_uncoated[i], nu_uncoated[i])
        K_uncoated.append(k)
        G_uncoated.append(g)
    for i in range(len(c_coated)):
        k, g = get_k_g_from_e_nu(E_coated[i], nu_coated[i])
        K_coated.append(k)
        G_coated.append(g)
    for i in range(len(c_coating)):
        k, g = get_k_g_from_e_nu(E_coating[i], nu_coating[i])
        K_coating.append(k)
        G_coating.append(g)

    # input processing
    n_u = len(c_uncoated)  # number of uncoated particles + matrix
    n_c = len(c_coated)  # number of coated particles
    n_total = 1+n_u+n_c*2

    c_total = [c_matrix] + c_uncoated + c_coated + c_coating   # normal MT homogenization w/ uncoated particles
    K_total = [K_matrix] + K_uncoated + K_coated + K_coating
    G_total = [G_matrix] + G_uncoated + G_coated + G_coating

    # constraint constants for uncoated particles
    alpha_0 = 3*K_matrix/(3*K_matrix+4*G_matrix)
    beta_0 = 6*(K_matrix+2*G_matrix)/(5*(3*K_matrix+4*G_matrix))

    # dilute concentration factors
    A_dil_K = np.zeros(n_total)
    A_dil_G = np.zeros(n_total)

    if sum(c_uncoated) > 0:
        for n in range(n_u):
            A_dil_K[n+1] = 1/(1+alpha_0*(K_uncoated[n]/K_matrix-1))
            A_dil_G[n+1] = 1/(1+beta_0*(G_uncoated[n]/G_matrix-1))

    if n_c > 0:
        for n in range(n_c):
            TK, TG = strain_factors_herve_zaoui(2, [K_coated[n], K_coating[n], K_matrix],
                                                [G_coated[n], G_coating[n], G_matrix],
                                                [nu_coated[n], nu_coating[n], nu_matrix],
                                                [r_coated[n], r_coating[n]])
            A_dil_K[1+n_u+n] = TK[0]
            A_dil_K[1+n_u+n_c+n] = TK[1]
            A_dil_G[1+n_u+n] = TG[0]
            A_dil_G[1+n_u+n_c+n] = TG[1]

    A_dil_K[0] = 1-sum(A_dil_K)
    A_dil_G[0] = 1-sum(A_dil_G)

    # Mori-Tanaka factors
    A_MT_K = c_matrix
    A_MT_G = c_matrix
    for i in range(1, n_total):
        A_MT_K = A_MT_K+c_total[i]*A_dil_K[i]
        A_MT_G = A_MT_G+c_total[i]*A_dil_G[i]
    A_MT_K = 1/A_MT_K
    A_MT_G = 1/A_MT_G

    # K_eff, G_eff
    K_eff = c_matrix*K_matrix
    for i in range(1, n_total):
        K_eff = K_eff+c_total[i]*K_total[i]*A_dil_K[i]
    K_eff = K_eff*A_MT_K

    G_eff = c_matrix*G_matrix
    for i in range(1, n_total):
        G_eff = G_eff+c_total[i]*G_total[i]*A_dil_G[i]
    G_eff = G_eff*A_MT_G

    return K_eff, G_eff, A_dil_K, A_dil_G, A_MT_K, A_MT_G
